//
//  FilterMaker.swift
//  CoreImageFiltersDemo
//
//  Created by Gabriel Theodoropoulos.
//  https://serialcoder.dev
//

import Foundation
import CoreImage
import UIKit

enum Filters: CaseIterable, CustomStringConvertible {
    case none, sepia, motionBlur, colorInvert, crystallize, comic
    
    var description: String {
        switch self {
            case .none: return "None"
            case .sepia: return "Sepia"
            case .motionBlur: return "Motion Blur"
            case .colorInvert: return "Color Invert"
            case .crystallize: return "Crystallize"
            case .comic: return "Comic Effect"
        }
    }
}


class FilterMaker: ObservableObject {
    @Published var image = UIImage()
    var originalImage: UIImage
    let context = CIContext()
    var ciImage: CIImage? {
        guard let cgImage = originalImage.cgImage else { return nil }
        return CIImage(cgImage: cgImage)
    }
    
    init() {
        originalImage = UIImage(named: "sample_image") ?? UIImage()
        image = originalImage
    }
    
    func removeFilter() {
        image = originalImage
    }
    
    func applySepia() {
        guard let ciImage else { return }
        let filter = CIFilter(name: "CISepiaTone")
        filter?.setValue(ciImage, forKey: "inputImage")
        filter?.setValue(0.9, forKey: "inputIntensity")
        getImage(usingFilter: filter)
    }
    
    func applyMotionBlur() {
        guard let ciImage else { return }
        let filter = CIFilter(name: "CIMotionBlur")
        filter?.setValue(ciImage, forKey: kCIInputImageKey)
        filter?.setValue(30.0, forKey: kCIInputRadiusKey)
        filter?.setValue(20.0, forKey: kCIInputAngleKey)
        getImage(usingFilter: filter)
    }
    
    func applyColorInvert() {
        guard let ciImage else { return }
        let filter = CIFilter(name: "CIColorInvert")
        filter?.setValue(ciImage, forKey: kCIInputImageKey)
        getImage(usingFilter: filter)
    }
    
    func applyCrystallize() {
        guard let ciImage else { return }
        let filter = CIFilter(name: "CICrystallize")
        filter?.setValue(ciImage, forKey: kCIInputImageKey)
        filter?.setValue(35, forKey: kCIInputRadiusKey)
        filter?.setValue(CIVector(x: 200, y: 200), forKey: kCIInputCenterKey)
        getImage(usingFilter: filter)
    }
    
    func applyComicEffect() {
        guard let ciImage else { return }
        let filter = CIFilter(name: "CIComicEffect")
        filter?.setValue(ciImage, forKey: kCIInputImageKey)
        getImage(usingFilter: filter)
    }
    
    fileprivate func getImage(usingFilter filter: CIFilter?) {
        guard let output = filter?.outputImage else { return }
        guard let cgImage = context.createCGImage(output, from: output.extent) else { return }
        image = UIImage(cgImage: cgImage)
    }
    
    
    func chainFilters() {
        guard let ciImage else { return }
        
        let sepiaFilter = CIFilter(name: "CISepiaTone")
        sepiaFilter?.setValue(ciImage, forKey: kCIInputImageKey)
        sepiaFilter?.setValue(0.9, forKey: kCIInputIntensityKey)
        
        guard let sepiaOutput = sepiaFilter?.outputImage else { return }
        
        let blurFilter = CIFilter(name: "CIMotionBlur")
        blurFilter?.setValue(sepiaOutput, forKey: kCIInputImageKey)
        blurFilter?.setValue(30.0, forKey: kCIInputRadiusKey)
        blurFilter?.setValue(20.0, forKey: kCIInputAngleKey)
        
        getImage(usingFilter: blurFilter)
    }
}
